function [GlobMin,GlobMinX] = acs0(fx,XLow,XHi,MaxPop,MaxIter,p)
% Implements the Artificial Cooperative Search (ACS) Optimization.
% 
% Code by Namir Shammas, February 2018.
%
% INPUT
% =====
% f - handle to the optimized function f(x).
% XLow - the array of lower limits.
% XHi - the array of upper limits. Note: You can supply a single-element
% array to this parameter. The function will adjust the array size to match
% that of array XLow and will have all elements in array XHi equal to the
% single value you supplied.
% MaxPop - the Population size.
% MaxIter - the maximum number of iterations
% p - probability of biological interaction in ACS
% 
% OUTPUT 
% ======
% 
% GlobMin - best value for f(x).
% GlobMinX - best solution vector.
%
% REFERENCE
% =========
% "Artificial cooperative search algorithm for numerical
% optimization problems", by Pinar Civicioglu
% Information Sciences 229 (2013) 5876
%

  if nargin < 6, p = 0.1; end
  if nargin < 5, MaxIter = 2000; end
  if nargin < 4, MaxPop = 200; end
  
  n = length(XLow);
  n2 = length(XHi);
  if n2 < n && n2 > 0
    XHi = XHi(1) + zeros(1,n);
  end
  PopAlpha = zeros(MaxPop,n);
  PopBeta = zeros(MaxPop,n);
  YAlpha = zeros(MaxPop,1);
  YBeta = zeros(MaxPop,1);
  
  for i=1:MaxPop
    PopAlpha(i,:) = XLow + (XHi - XLow) .* rand(1,n);
    PopBeta(i,:) = XLow + (XHi - XLow) .* rand(1,n);
    YAlpha(i) = fx(PopAlpha(i,:));
    YBeta(i) = fx(PopAlpha(i,:));
  end
 
  GlobMin = 1e+99;
  
  % -------------------------------------- Main Loop
  for iGen=1:MaxIter  
    if rand < rand
      Predator = PopAlpha;
      ypred = YAlpha;
      key = 1;
    else
      Predator = PopBeta;
      ypred = YBeta;
      key = 2;
    end
    
    if rand < rand
      Prey = PopAlpha(:,1:n);
    else
      Prey = PopBeta(:,1:n);
    end
    Prey = Prey(randperm(MaxPop),1:n);
    if rand < rand
      R = 4 * rand *(rand - rand);
    else
      R = gampdf(4 * rand, 1);
    end
    
    M = 1 + zeros(MaxPop,n);
    
    for q=1:MaxPop * n
      if rand < p * rand
        i = 1 + fix(rand * MaxPop);
        j = 1 + fix(rand * n);
        M(i,j) = 0;
      end
    end
    
    if rand < p * rand
        for i=1:MaxPop
          for j=1:n
            if rand < p * rand
              M(i,j) = 1;
            else
              M(i,j) = 0;
            end
          end
        end
    end
    
    for i=1:MaxPop
      if sum(M(i,:)) == n
        j = 1 + fix(rand * n);
        M(i,j) = 0;
      end
    end
    
    % mutation
    x = Predator + R * (Prey - Predator);
    for i=1:MaxPop
      for j=1:n
        % crossover
        if M(i,j) > 0
          x(i,j) = Predator(i,j);
        end
        if x(i,j) < XLow(j) || x(i,j) > XHi(j)
          x(i,j) = XLow(j) + (XHi(j) - XLow(j)) * rand;
        end
      end
    end
    
    % selection
    for i=1:MaxPop
      zFx = fx(x(i,:));
      if zFx < ypred(i)
        Predator(i,:) = x(i,:);
        ypred(i) = zFx;
      end
    end
    
    if key==1
      PopAlpha = Predator;
      YAlpha = ypred;
      [Amin,IdxA] = min(YAlpha);
      ybest = Amin;
      bestX = PopAlpha(IdxA,:);
    else
      PopBeta = Predator;
      YBeta = ypred;    
      [Bmin,IdxB] = min(YBeta);
      ybest = Bmin;
      bestX = PopBeta(IdxB,1:n);
    end
    
    if ybest < GlobMin
      GlobMin = ybest;
      GlobMinX = bestX;
    end
  end % iGen
end


